@@ -1,6 +1,6 @@
 /* BFD back-end for VERSAdos-E objects.
-   Copyright 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003, 2004
-   Free Software Foundation, Inc.
+   Copyright 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+   2006, 2007 Free Software Foundation, Inc.
    Written by Steve Chamberlain of Cygnus Support <sac@cygnus.com>.
 
    Versados is a Motorola trademark.
@@ -9,7 +9,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -19,7 +19,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston,
+   MA 02110-1301, USA.  */
 
 /*
    SUBSECTION
@@ -35,79 +36,53 @@
    o Identification Record
    o External Symbol Definition Record
    o Object Text Record
-   o End Record
+   o End Record.  */
 
- */
-
-#include "bfd.h"
 #include "sysdep.h"
+#include "bfd.h"
 #include "libbfd.h"
 #include "libiberty.h"
 
-static bfd_boolean versados_mkobject PARAMS ((bfd *));
-static bfd_boolean versados_scan PARAMS ((bfd *));
-static const bfd_target *versados_object_p PARAMS ((bfd *));
-static asymbol *versados_new_symbol
-  PARAMS ((bfd *, int, const char *, bfd_vma, asection *));
-static char *new_symbol_string PARAMS ((bfd *, const char *));
-static const bfd_target *versados_object_p PARAMS ((bfd *));
-static bfd_boolean versados_pass_2 PARAMS ((bfd *));
-static bfd_boolean versados_get_section_contents
-  PARAMS ((bfd *, asection *, void *, file_ptr, bfd_size_type));
-static bfd_boolean versados_set_section_contents
-  PARAMS ((bfd *, sec_ptr, const void *, file_ptr, bfd_size_type));
-static int versados_sizeof_headers PARAMS ((bfd *, bfd_boolean));
-static long int versados_get_symtab_upper_bound PARAMS ((bfd *));
-static long int versados_canonicalize_symtab PARAMS ((bfd *, asymbol **));
-static void versados_get_symbol_info
-  PARAMS ((bfd *, asymbol *, symbol_info *));
-static void versados_print_symbol
-  PARAMS ((bfd *, PTR, asymbol *, bfd_print_symbol_type));
-static long versados_get_reloc_upper_bound
-  PARAMS ((bfd *, sec_ptr));
-static long versados_canonicalize_reloc
-  PARAMS ((bfd *, sec_ptr, arelent **, asymbol **));
 
 #define VHEADER '1'
 #define VESTDEF '2'
 #define VOTR '3'
 #define VEND '4'
 
-#define ES_BASE 17		/* first symbol has esdid 17 */
+#define ES_BASE 17		/* First symbol has esdid 17.  */
 
-/* Per file target dependent information */
+/* Per file target dependent information.  */
 
-/* one for each section */
+/* One for each section.  */
 struct esdid
-  {
-    asection *section;		/* ptr to bfd version */
-    unsigned char *contents;	/* used to build image */
-    int pc;
-    int relocs;			/* reloc count, valid end of pass 1 */
-    int donerel;		/* have relocs been translated */
-  };
+{
+  asection *section;		/* Ptr to bfd version.  */
+  unsigned char *contents;	/* Used to build image.  */
+  int pc;
+  int relocs;			/* Reloc count, valid end of pass 1.  */
+  int donerel;			/* Have relocs been translated.  */
+};
 
 typedef struct versados_data_struct
-  {
-    int es_done;		/* count of symbol index, starts at ES_BASE */
-    asymbol *symbols;		/* pointer to local symbols */
-    char *strings;		/* strings of all the above */
-    int stringlen;		/* len of string table (valid end of pass1) */
-    int nsecsyms;		/* number of sections */
+{
+  int es_done;			/* Count of symbol index, starts at ES_BASE.  */
+  asymbol *symbols;		/* Pointer to local symbols.  */
+  char *strings;		/* Strings of all the above.  */
+  int stringlen;		/* Len of string table (valid end of pass1).  */
+  int nsecsyms;			/* Number of sections.  */
 
-    int ndefs;			/* number of exported symbols (they dont get esdids) */
-    int nrefs;			/* number of imported symbols  (valid end of pass1) */
+  int ndefs;			/* Number of exported symbols (they dont get esdids).  */
+  int nrefs;			/* Number of imported symbols  (valid end of pass1).  */
 
-    int ref_idx;		/* current processed value of the above */
-    int def_idx;
+  int ref_idx;			/* Current processed value of the above.  */
+  int def_idx;
 
-    int pass_2_done;
+  int pass_2_done;
 
-    struct esdid e[16];		/* per section info */
-    int alert;			/* to see if we're trampling */
-    asymbol *rest[256 - 16];	/* per symbol info */
-
-  }
+  struct esdid e[16];		/* Per section info.  */
+  int alert;			/* To see if we're trampling.  */
+  asymbol *rest[256 - 16];	/* Per symbol info.  */
+}
 tdata_type;
 
 #define VDATA(abfd)       (abfd->tdata.versados_data)
@@ -115,72 +90,67 @@ tdata_type;
 #define RDATA(abfd, n)    (abfd->tdata.versados_data->rest[n])
 
 struct ext_otr
-  {
-    unsigned char size;
-    char type;
-    unsigned char map[4];
-    unsigned char esdid;
-    unsigned char data[200];
-  };
+{
+  unsigned char size;
+  char type;
+  unsigned char map[4];
+  unsigned char esdid;
+  unsigned char data[200];
+};
 
 struct ext_vheader
-  {
-    unsigned char size;
-    char type;			/* record type */
-    char name[10];		/* module name */
-    char rev;			/* module rev number */
-    char lang;
-    char vol[4];
-    char user[2];
-    char cat[8];
-    char fname[8];
-    char ext[2];
-    char time[3];
-    char date[3];
-    char rest[211];
-  };
+{
+  unsigned char size;
+  char type;			/* Record type.  */
+  char name[10];		/* Module name.  */
+  char rev;			/* Module rev number.  */
+  char lang;
+  char vol[4];
+  char user[2];
+  char cat[8];
+  char fname[8];
+  char ext[2];
+  char time[3];
+  char date[3];
+  char rest[211];
+};
 
 struct ext_esd
-  {
-    unsigned char size;
-    char type;
-    unsigned char esd_entries[1];
-  };
-#define ESD_ABS 	0
-#define ESD_COMMON 	1
-#define ESD_STD_REL_SEC 2
-#define ESD_SHRT_REL_SEC 3
-#define ESD_XDEF_IN_SEC 4
-#define ESD_XREF_SYM    7
-#define ESD_XREF_SEC	6
-#define ESD_XDEF_IN_ABS 5
-union ext_any
-  {
-    unsigned char size;
-    struct ext_vheader header;
-    struct ext_esd esd;
-    struct ext_otr otr;
-  };
+{
+  unsigned char size;
+  char type;
+  unsigned char esd_entries[1];
+};
 
-static int get_record PARAMS ((bfd *, union ext_any *));
-static int get_4 PARAMS ((unsigned char **));
-static void get_10 PARAMS ((unsigned char **, char *));
-static void process_esd PARAMS ((bfd *, struct ext_esd *, int));
-static int get_offset PARAMS ((int, unsigned char *));
-static void process_otr PARAMS ((bfd *, struct ext_otr *, int));
+#define ESD_ABS 	  0
+#define ESD_COMMON 	  1
+#define ESD_STD_REL_SEC   2
+#define ESD_SHRT_REL_SEC  3
+#define ESD_XDEF_IN_SEC   4
+#define ESD_XDEF_IN_ABS   5
+#define ESD_XREF_SEC	  6
+#define ESD_XREF_SYM      7
+
+union ext_any
+{
+  unsigned char size;
+  struct ext_vheader header;
+  struct ext_esd esd;
+  struct ext_otr otr;
+};
 
 /* Initialize by filling in the hex conversion array.  */
 
 /* Set up the tdata information.  */
 
 static bfd_boolean
-versados_mkobject (abfd)
-     bfd *abfd;
+versados_mkobject (bfd *abfd)
 {
   if (abfd->tdata.versados_data == NULL)
     {
       bfd_size_type amt = sizeof (tdata_type);
-      tdata_type *tdata = (tdata_type *) bfd_alloc (abfd, amt);
+      tdata_type *tdata = bfd_alloc (abfd, amt);
+
       if (tdata == NULL)
 	return FALSE;
       abfd->tdata.versados_data = tdata;
@@ -197,12 +167,11 @@ versados_mkobject (abfd)
    error messages.  */
 
 static asymbol *
-versados_new_symbol (abfd, snum, name, val, sec)
-     bfd *abfd;
-     int snum;
-     const char *name;
-     bfd_vma val;
-     asection *sec;
+versados_new_symbol (bfd *abfd,
+		     int snum,
+		     const char *name,
+		     bfd_vma val,
+		     asection *sec)
 {
   asymbol *n = VDATA (abfd)->symbols + snum;
   n->name = name;
@@ -214,9 +183,7 @@ versados_new_symbol (abfd, snum, name, val, sec)
 }
 
 static int
-get_record (abfd, ptr)
-     bfd *abfd;
-     union ext_any *ptr;
+get_record (bfd *abfd, union ext_any *ptr)
 {
   if (bfd_bread (&ptr->size, (bfd_size_type) 1, abfd) != 1
       || (bfd_bread ((char *) ptr + 1, (bfd_size_type) ptr->size, abfd)
@@ -226,24 +193,22 @@ get_record (abfd, ptr)
 }
 
 static int
-get_4 (pp)
-     unsigned char **pp;
+get_4 (unsigned char **pp)
 {
   unsigned char *p = *pp;
+
   *pp += 4;
   return (p[0] << 24) | (p[1] << 16) | (p[2] << 8) | (p[3] << 0);
 }
 
 static void
-get_10 (pp, name)
-     unsigned char **pp;
-     char *name;
+get_10 (unsigned char **pp, char *name)
 {
   char *p = (char *) *pp;
   int len = 10;
+
   *pp += len;
-  while (*p != ' '
-	 && len)
+  while (*p != ' ' && len)
     {
       *name++ = *p++;
       len--;
@@ -252,23 +217,19 @@ get_10 (pp, name)
 }
 
 static char *
-new_symbol_string (abfd, name)
-     bfd *abfd;
-     const char *name;
+new_symbol_string (bfd *abfd, const char *name)
 {
   char *n = VDATA (abfd)->strings;
+
   strcpy (VDATA (abfd)->strings, name);
   VDATA (abfd)->strings += strlen (VDATA (abfd)->strings) + 1;
   return n;
 }
 
 static void
-process_esd (abfd, esd, pass)
-     bfd *abfd;
-     struct ext_esd *esd;
-     int pass;
+process_esd (bfd *abfd, struct ext_esd *esd, int pass)
 {
-  /* Read through the ext def for the est entries */
+  /* Read through the ext def for the est entries.  */
   int togo = esd->size - 2;
   bfd_vma size;
   bfd_vma start;
@@ -276,17 +237,19 @@ process_esd (abfd, esd, pass)
   char name[11];
   unsigned char *ptr = esd->esd_entries;
   unsigned char *end = ptr + togo;
+
   while (ptr < end)
     {
       int scn = *ptr & 0xf;
       int typ = (*ptr >> 4) & 0xf;
 
-      /* Declare this section */
+      /* Declare this section.  */
       sprintf (name, "%d", scn);
       sec = bfd_make_section_old_way (abfd, strdup (name));
       sec->target_index = scn;
       EDATA (abfd, scn).section = sec;
       ptr++;
+
       switch (typ)
 	{
 	default:
@@ -297,14 +260,13 @@ process_esd (abfd, esd, pass)
 	    int snum = VDATA (abfd)->ref_idx++;
 	    get_10 (&ptr, name);
 	    if (pass == 1)
-	      {
-		VDATA (abfd)->stringlen += strlen (name) + 1;
-	      }
+	      VDATA (abfd)->stringlen += strlen (name) + 1;
 	    else
 	      {
 		int esidx;
 		asymbol *s;
 		char *n = new_symbol_string (abfd, name);
+
 		s = versados_new_symbol (abfd, snum, n, (bfd_vma) 0,
 					 bfd_und_section_ptr);
 		esidx = VDATA (abfd)->es_done++;
@@ -319,10 +281,8 @@ process_esd (abfd, esd, pass)
 	  break;
 	case ESD_STD_REL_SEC:
 	case ESD_SHRT_REL_SEC:
-	  {
-	    sec->size = get_4 (&ptr);
-	    sec->flags |= SEC_ALLOC;
-	  }
+	  sec->size = get_4 (&ptr);
+	  sec->flags |= SEC_ALLOC;
 	  break;
 	case ESD_XDEF_IN_ABS:
 	  sec = (asection *) & bfd_abs_section;
@@ -330,17 +290,17 @@ process_esd (abfd, esd, pass)
 	  {
 	    int snum = VDATA (abfd)->def_idx++;
 	    bfd_vma val;
+
 	    get_10 (&ptr, name);
 	    val = get_4 (&ptr);
 	    if (pass == 1)
-	      {
-		/* Just remember the symbol */
-		VDATA (abfd)->stringlen += strlen (name) + 1;
-	      }
+	      /* Just remember the symbol.  */
+	      VDATA (abfd)->stringlen += strlen (name) + 1;
 	    else
 	      {
 		asymbol *s;
 		char *n = new_symbol_string (abfd, name);
+
 		s = versados_new_symbol (abfd, snum + VDATA (abfd)->nrefs, n,
 					 val, sec);
 		s->flags |= BSF_GLOBAL;
@@ -351,8 +311,8 @@ process_esd (abfd, esd, pass)
     }
 }
 
-#define R_RELWORD 1
-#define R_RELLONG 2
+#define R_RELWORD     1
+#define R_RELLONG     2
 #define R_RELWORD_NEG 3
 #define R_RELLONG_NEG 4
 
@@ -374,14 +334,14 @@ reloc_howto_type versados_howto_table[] =
 };
 
 static int
-get_offset (len, ptr)
-     int len;
-     unsigned char *ptr;
+get_offset (int len, unsigned char *ptr)
 {
   int val = 0;
+
   if (len)
     {
       int i;
+
       val = *ptr++;
       if (val & 0x80)
 	val |= ~0xff;
@@ -393,10 +353,7 @@ get_offset (len, ptr)
 }
 
 static void
-process_otr (abfd, otr, pass)
-     bfd *abfd;
-     struct ext_otr *otr;
-     int pass;
+process_otr (bfd *abfd, struct ext_otr *otr, int pass)
 {
   unsigned long shift;
   unsigned char *srcp = otr->data;
@@ -423,13 +380,14 @@ process_otr (abfd, otr, pass)
 
 	  if (esdids == 0)
 	    {
-	      /* A zero esdid means the new pc is the offset given */
+	      /* A zero esdid means the new pc is the offset given.  */
 	      dst_idx += get_offset (offsetlen, srcp);
 	      srcp += offsetlen;
 	    }
 	  else
 	    {
 	      int val = get_offset (offsetlen, srcp + esdids);
+
 	      if (pass == 1)
 		need_contents = 1;
 	      else
@@ -446,10 +404,11 @@ process_otr (abfd, otr, pass)
 		  if (esdid)
 		    {
 		      int rn = EDATA (abfd, otr->esdid - 1).relocs++;
+
 		      if (pass == 1)
 			{
-			  /* this is the first pass over the data,
-			     just remember that we need a reloc */
+			  /* This is the first pass over the data,
+			     just remember that we need a reloc.  */
 			}
 		      else
 			{
@@ -457,7 +416,7 @@ process_otr (abfd, otr, pass)
 			  EDATA (abfd, otr->esdid - 1).section->relocation + rn;
 			  n->address = dst_idx;
 
-			  n->sym_ptr_ptr = (asymbol **) esdid;
+			  n->sym_ptr_ptr = (asymbol **) (size_t) esdid;
 			  n->addend = 0;
 			  n->howto = versados_howto_table + ((j & 1) * 2) + (sizeinwords - 1);
 			}
@@ -473,7 +432,7 @@ process_otr (abfd, otr, pass)
 	  if (dst_idx < esdid->section->size)
 	    if (pass == 2)
 	      {
-		/* absolute code, comes in 16 bit lumps */
+		/* Absolute code, comes in 16 bit lumps.  */
 		contents[dst_idx] = srcp[0];
 		contents[dst_idx + 1] = srcp[1];
 	      }
@@ -486,13 +445,12 @@ process_otr (abfd, otr, pass)
   if (!contents && need_contents)
     {
       bfd_size_type size = esdid->section->size;
-      esdid->contents = (unsigned char *) bfd_alloc (abfd, size);
+      esdid->contents = bfd_alloc (abfd, size);
     }
 }
 
 static bfd_boolean
-versados_scan (abfd)
-     bfd *abfd;
+versados_scan (bfd *abfd)
 {
   int loop = 1;
   int i;
@@ -510,6 +468,7 @@ versados_scan (abfd)
   while (loop)
     {
       union ext_any any;
+
       if (!get_record (abfd, &any))
 	return TRUE;
       switch (any.header.type)
@@ -528,8 +487,7 @@ versados_scan (abfd)
 	}
     }
 
-  /* Now allocate space for the relocs and sections */
-
+  /* Now allocate space for the relocs and sections.  */
   VDATA (abfd)->nrefs = VDATA (abfd)->ref_idx;
   VDATA (abfd)->ndefs = VDATA (abfd)->def_idx;
   VDATA (abfd)->ref_idx = 0;
@@ -540,10 +498,11 @@ versados_scan (abfd)
   for (i = 0; i < 16; i++)
     {
       struct esdid *esdid = &EDATA (abfd, i);
+
       if (esdid->section)
 	{
 	  amt = (bfd_size_type) esdid->relocs * sizeof (arelent);
-	  esdid->section->relocation = (arelent *) bfd_alloc (abfd, amt);
+	  esdid->section->relocation = bfd_alloc (abfd, amt);
 
 	  esdid->pc = 0;
 
@@ -556,7 +515,7 @@ versados_scan (abfd)
 
 	  esdid->relocs = 0;
 
-	  /* Add an entry into the symbol table for it */
+	  /* Add an entry into the symbol table for it.  */
 	  nsecs++;
 	  VDATA (abfd)->stringlen += strlen (esdid->section->name) + 1;
 	}
@@ -566,7 +525,7 @@ versados_scan (abfd)
 
   amt = abfd->symcount;
   amt *= sizeof (asymbol);
-  VDATA (abfd)->symbols = (asymbol *) bfd_alloc (abfd, amt);
+  VDATA (abfd)->symbols = bfd_alloc (abfd, amt);
 
   amt = VDATA (abfd)->stringlen;
   VDATA (abfd)->strings = bfd_alloc (abfd, amt);
@@ -576,12 +535,12 @@ versados_scan (abfd)
     return FALSE;
 
   /* Actually fill in the section symbols,
-     we stick them at the end of the table */
-
+     we stick them at the end of the table.  */
   for (j = VDATA (abfd)->nrefs + VDATA (abfd)->ndefs, i = 0; i < 16; i++)
     {
       struct esdid *esdid = &EDATA (abfd, i);
       asection *sec = esdid->section;
+
       if (sec)
 	{
 	  asymbol *s = VDATA (abfd)->symbols + j;
@@ -593,11 +552,12 @@ versados_scan (abfd)
 	  j++;
 	}
     }
+
   if (abfd->symcount)
     abfd->flags |= HAS_SYMS;
 
   /* Set this to nsecs - since we've already planted the section
-     symbols */
+     symbols.  */
   VDATA (abfd)->nsecsyms = nsecs;
 
   VDATA (abfd)->ref_idx = 0;
@@ -608,8 +568,7 @@ versados_scan (abfd)
 /* Check whether an existing file is a versados  file.  */
 
 static const bfd_target *
-versados_object_p (abfd)
-     bfd *abfd;
+versados_object_p (bfd *abfd)
 {
   struct ext_vheader ext;
   unsigned char len;
@@ -643,7 +602,6 @@ versados_object_p (abfd)
     }
 
   /* OK, looks like a record, build the tdata and read in.  */
-
   tdata_save = abfd->tdata.versados_data;
   if (!versados_mkobject (abfd) || !versados_scan (abfd))
     {
@@ -655,8 +613,7 @@ versados_object_p (abfd)
 }
 
 static bfd_boolean
-versados_pass_2 (abfd)
-     bfd *abfd;
+versados_pass_2 (bfd *abfd)
 {
   union ext_any any;
 
@@ -668,8 +625,7 @@ versados_pass_2 (abfd)
 
   VDATA (abfd)->es_done = ES_BASE;
 
-  /* read records till we get to where we want to be */
-
+  /* Read records till we get to where we want to be.  */
   while (1)
     {
       get_record (abfd, &any);
@@ -689,12 +645,11 @@ versados_pass_2 (abfd)
 }
 
 static bfd_boolean
-versados_get_section_contents (abfd, section, location, offset, count)
-     bfd *abfd;
-     asection *section;
-     PTR location;
-     file_ptr offset;
-     bfd_size_type count;
+versados_get_section_contents (bfd *abfd,
+			       asection *section,
+			       void * location,
+			       file_ptr offset,
+			       bfd_size_type count)
 {
   if (!versados_pass_2 (abfd))
     return FALSE;
@@ -710,20 +665,18 @@ versados_get_section_contents (abfd, section, location, offset, count)
   _bfd_generic_get_section_contents_in_window
 
 static bfd_boolean
-versados_set_section_contents (abfd, section, location, offset, bytes_to_do)
-     bfd *abfd ATTRIBUTE_UNUSED;
-     sec_ptr section ATTRIBUTE_UNUSED;
-     const PTR location ATTRIBUTE_UNUSED;
-     file_ptr offset ATTRIBUTE_UNUSED;
-     bfd_size_type bytes_to_do ATTRIBUTE_UNUSED;
+versados_set_section_contents (bfd *abfd ATTRIBUTE_UNUSED,
+			       sec_ptr section ATTRIBUTE_UNUSED,
+			       const void * location ATTRIBUTE_UNUSED,
+			       file_ptr offset ATTRIBUTE_UNUSED,
+			       bfd_size_type bytes_to_do ATTRIBUTE_UNUSED)
 {
   return FALSE;
 }
 
 static int
-versados_sizeof_headers (abfd, exec)
-     bfd *abfd ATTRIBUTE_UNUSED;
-     bfd_boolean exec ATTRIBUTE_UNUSED;
+versados_sizeof_headers (bfd *abfd ATTRIBUTE_UNUSED,
+			 struct bfd_link_info *info ATTRIBUTE_UNUSED)
 {
   return 0;
 }
@@ -731,8 +684,7 @@ versados_sizeof_headers (abfd, exec)
 /* Return the amount of memory needed to read the symbol table.  */
 
 static long
-versados_get_symtab_upper_bound (abfd)
-     bfd *abfd;
+versados_get_symtab_upper_bound (bfd *abfd)
 {
   return (bfd_get_symcount (abfd) + 1) * sizeof (asymbol *);
 }
@@ -740,9 +692,7 @@ versados_get_symtab_upper_bound (abfd)
 /* Return the symbol table.  */
 
 static long
-versados_canonicalize_symtab (abfd, alocation)
-     bfd *abfd;
-     asymbol **alocation;
+versados_canonicalize_symtab (bfd *abfd, asymbol **alocation)
 {
   unsigned int symcount = bfd_get_symcount (abfd);
   unsigned int i;
@@ -753,9 +703,7 @@ versados_canonicalize_symtab (abfd, alocation)
   for (i = 0, s = VDATA (abfd)->symbols;
        i < symcount;
        s++, i++)
-    {
-      *alocation++ = s;
-    }
+    *alocation++ = s;
 
   *alocation = NULL;
 
@@ -763,50 +711,46 @@ versados_canonicalize_symtab (abfd, alocation)
 }
 
 static void
-versados_get_symbol_info (ignore_abfd, symbol, ret)
-     bfd *ignore_abfd ATTRIBUTE_UNUSED;
-     asymbol *symbol;
-     symbol_info *ret;
+versados_get_symbol_info (bfd *abfd ATTRIBUTE_UNUSED,
+			  asymbol *symbol,
+			  symbol_info *ret)
 {
   bfd_symbol_info (symbol, ret);
 }
 
 static void
-versados_print_symbol (abfd, afile, symbol, how)
-     bfd *abfd;
-     PTR afile;
-     asymbol *symbol;
-     bfd_print_symbol_type how;
+versados_print_symbol (bfd *abfd,
+		       void * afile,
+		       asymbol *symbol,
+		       bfd_print_symbol_type how)
 {
   FILE *file = (FILE *) afile;
+
   switch (how)
     {
     case bfd_print_symbol_name:
       fprintf (file, "%s", symbol->name);
       break;
     default:
-      bfd_print_symbol_vandf (abfd, (PTR) file, symbol);
+      bfd_print_symbol_vandf (abfd, (void *) file, symbol);
       fprintf (file, " %-5s %s",
 	       symbol->section->name,
 	       symbol->name);
-
     }
 }
 
 static long
-versados_get_reloc_upper_bound (abfd, asect)
-     bfd *abfd ATTRIBUTE_UNUSED;
-     sec_ptr asect;
+versados_get_reloc_upper_bound (bfd *abfd ATTRIBUTE_UNUSED,
+				sec_ptr asect)
 {
   return (asect->reloc_count + 1) * sizeof (arelent *);
 }
 
 static long
-versados_canonicalize_reloc (abfd, section, relptr, symbols)
-     bfd *abfd;
-     sec_ptr section;
-     arelent **relptr;
-     asymbol **symbols;
+versados_canonicalize_reloc (bfd *abfd,
+			     sec_ptr section,
+			     arelent **relptr,
+			     asymbol **symbols)
 {
   unsigned int count;
   arelent *src;
@@ -816,99 +760,85 @@ versados_canonicalize_reloc (abfd, section, relptr, symbols)
   if (!EDATA (abfd, section->target_index).donerel)
     {
       EDATA (abfd, section->target_index).donerel = 1;
-      /* translate from indexes to symptr ptrs */
+      /* Translate from indexes to symptr ptrs.  */
       for (count = 0; count < section->reloc_count; count++)
 	{
-	  int esdid = (int) src[count].sym_ptr_ptr;
+	  int esdid = (int) (size_t) src[count].sym_ptr_ptr;
 
 	  if (esdid == 0)
+	    src[count].sym_ptr_ptr = bfd_abs_section.symbol_ptr_ptr;
+	  else if (esdid < ES_BASE)
 	    {
-	      src[count].sym_ptr_ptr = bfd_abs_section.symbol_ptr_ptr;
-	    }
-	  else if (esdid < ES_BASE)	/* Section relative thing */
-	    {
+	      /* Section relative thing.  */
 	      struct esdid *e = &EDATA (abfd, esdid - 1);
-	      if (!section)
-		{
-		 /** relocation relative to section which was
-		   never declared ! */
-		}
+
 	      src[count].sym_ptr_ptr = e->section->symbol_ptr_ptr;
 	    }
 	  else
-	    {
-	      src[count].sym_ptr_ptr = symbols + esdid - ES_BASE;
-	    }
-
+	    src[count].sym_ptr_ptr = symbols + esdid - ES_BASE;
 	}
     }
 
   for (count = 0; count < section->reloc_count; count++)
-    {
-      *relptr++ = src++;
-    }
+    *relptr++ = src++;
+
   *relptr = 0;
   return section->reloc_count;
 }
 
-#define	versados_close_and_cleanup _bfd_generic_close_and_cleanup
-#define versados_bfd_free_cached_info _bfd_generic_bfd_free_cached_info
-#define versados_new_section_hook _bfd_generic_new_section_hook
-
-#define versados_bfd_is_target_special_symbol \
-  ((bfd_boolean (*) (bfd *, asymbol *)) bfd_false)
-#define versados_bfd_is_local_label_name bfd_generic_is_local_label_name
-#define versados_get_lineno _bfd_nosymbols_get_lineno
-#define versados_find_nearest_line _bfd_nosymbols_find_nearest_line
-#define versados_make_empty_symbol _bfd_generic_make_empty_symbol
-#define versados_bfd_make_debug_symbol _bfd_nosymbols_bfd_make_debug_symbol
-#define versados_read_minisymbols _bfd_generic_read_minisymbols
-#define versados_minisymbol_to_symbol _bfd_generic_minisymbol_to_symbol
-
-#define versados_bfd_reloc_type_lookup _bfd_norelocs_bfd_reloc_type_lookup
-
-#define versados_set_arch_mach bfd_default_set_arch_mach
-
-#define versados_bfd_get_relocated_section_contents \
-  bfd_generic_get_relocated_section_contents
-#define versados_bfd_relax_section bfd_generic_relax_section
-#define versados_bfd_gc_sections bfd_generic_gc_sections
-#define versados_bfd_merge_sections bfd_generic_merge_sections
-#define versados_bfd_is_group_section bfd_generic_is_group_section
-#define versados_bfd_discard_group bfd_generic_discard_group
-#define versados_section_already_linked \
-  _bfd_generic_section_already_linked
-#define versados_bfd_link_hash_table_create _bfd_generic_link_hash_table_create
-#define versados_bfd_link_hash_table_free _bfd_generic_link_hash_table_free
-#define versados_bfd_link_add_symbols _bfd_generic_link_add_symbols
-#define versados_bfd_link_just_syms _bfd_generic_link_just_syms
-#define versados_bfd_final_link _bfd_generic_final_link
-#define versados_bfd_link_split_section _bfd_generic_link_split_section
+#define	versados_close_and_cleanup                    _bfd_generic_close_and_cleanup
+#define versados_bfd_free_cached_info                 _bfd_generic_bfd_free_cached_info
+#define versados_new_section_hook                     _bfd_generic_new_section_hook
+#define versados_bfd_is_target_special_symbol   ((bfd_boolean (*) (bfd *, asymbol *)) bfd_false)
+#define versados_bfd_is_local_label_name              bfd_generic_is_local_label_name
+#define versados_get_lineno                           _bfd_nosymbols_get_lineno
+#define versados_find_nearest_line                    _bfd_nosymbols_find_nearest_line
+#define versados_find_inliner_info                    _bfd_nosymbols_find_inliner_info
+#define versados_make_empty_symbol                    _bfd_generic_make_empty_symbol
+#define versados_bfd_make_debug_symbol                _bfd_nosymbols_bfd_make_debug_symbol
+#define versados_read_minisymbols                     _bfd_generic_read_minisymbols
+#define versados_minisymbol_to_symbol                 _bfd_generic_minisymbol_to_symbol
+#define versados_bfd_reloc_type_lookup                _bfd_norelocs_bfd_reloc_type_lookup
+#define versados_bfd_reloc_name_lookup          _bfd_norelocs_bfd_reloc_name_lookup
+#define versados_set_arch_mach                        bfd_default_set_arch_mach
+#define versados_bfd_get_relocated_section_contents   bfd_generic_get_relocated_section_contents
+#define versados_bfd_relax_section                    bfd_generic_relax_section
+#define versados_bfd_gc_sections                      bfd_generic_gc_sections
+#define versados_bfd_merge_sections                   bfd_generic_merge_sections
+#define versados_bfd_is_group_section                 bfd_generic_is_group_section
+#define versados_bfd_discard_group                    bfd_generic_discard_group
+#define versados_section_already_linked               _bfd_generic_section_already_linked
+#define versados_bfd_link_hash_table_create           _bfd_generic_link_hash_table_create
+#define versados_bfd_link_hash_table_free             _bfd_generic_link_hash_table_free
+#define versados_bfd_link_add_symbols                 _bfd_generic_link_add_symbols
+#define versados_bfd_link_just_syms                   _bfd_generic_link_just_syms
+#define versados_bfd_final_link                       _bfd_generic_final_link
+#define versados_bfd_link_split_section               _bfd_generic_link_split_section
 
 const bfd_target versados_vec =
 {
-  "versados",			/* name */
+  "versados",			/* Name.  */
   bfd_target_versados_flavour,
-  BFD_ENDIAN_BIG,		/* target byte order */
-  BFD_ENDIAN_BIG,		/* target headers byte order */
-  (HAS_RELOC | EXEC_P |		/* object flags */
+  BFD_ENDIAN_BIG,		/* Target byte order.  */
+  BFD_ENDIAN_BIG,		/* Target headers byte order.  */
+  (HAS_RELOC | EXEC_P |		/* Object flags.  */
    HAS_LINENO | HAS_DEBUG |
    HAS_SYMS | HAS_LOCALS | WP_TEXT | D_PAGED),
   (SEC_CODE | SEC_DATA | SEC_ROM | SEC_HAS_CONTENTS
-   | SEC_ALLOC | SEC_LOAD | SEC_RELOC),		/* section flags */
-  0,				/* leading underscore */
-  ' ',				/* ar_pad_char */
-  16,				/* ar_max_namelen */
+   | SEC_ALLOC | SEC_LOAD | SEC_RELOC),		/* Section flags.  */
+  0,				/* Leading underscore.  */
+  ' ',				/* AR_pad_char.  */
+  16,				/* AR_max_namelen.  */
   bfd_getb64, bfd_getb_signed_64, bfd_putb64,
   bfd_getb32, bfd_getb_signed_32, bfd_putb32,
-  bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* data */
+  bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* Data.  */
   bfd_getb64, bfd_getb_signed_64, bfd_putb64,
   bfd_getb32, bfd_getb_signed_32, bfd_putb32,
-  bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* hdrs */
+  bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* Headers.  */
 
   {
     _bfd_dummy_target,
-    versados_object_p,		/* bfd_check_format */
+    versados_object_p,		/* bfd_check_format.  */
     _bfd_dummy_target,
     _bfd_dummy_target,
   },
@@ -918,7 +848,7 @@ const bfd_target versados_vec =
     _bfd_generic_mkarchive,
     bfd_false,
   },
-  {				/* bfd_write_contents */
+  {				/* bfd_write_contents.  */
     bfd_false,
     bfd_false,
     _bfd_write_archive_contents,
@@ -937,5 +867,5 @@ const bfd_target versados_vec =
 
   NULL,
 
-  (PTR) 0
+  NULL
 };
