@@ -1,11 +1,12 @@
 /* Native-dependent definitions for NetBSD.
-   Copyright 1994, 1996, 1999 Free Software Foundation, Inc.
+
+   Copyright 1994, 1996, 1999, 2004, 2007 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -14,12 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
-
-#define PTRACE_ARG3_TYPE char*
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #define FETCH_INFERIOR_REGISTERS
 
-#include "solib.h"		/* Support for shared libraries. */
