@@ -1,13 +1,13 @@
 /* Machine independent GDB support for core files on systems using "regsets".
 
-   Copyright (C) 1993, 1994, 1995, 1996, 1998, 1999, 2000, 2003
+   Copyright (C) 1993, 1994, 1995, 1996, 1998, 1999, 2000, 2003, 2007
    Free Software Foundation, Inc.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -16,9 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* This file is used by most systems that use ELF for their core
    dumps.  This includes most systems that have SVR4-ish variant of
@@ -34,6 +32,7 @@
 #include "gdbcore.h"
 #include "inferior.h"
 #include "target.h"
+#include "regcache.h"
 
 #include <fcntl.h>
 #include <errno.h>
@@ -59,11 +58,14 @@
    REG_ADDR is ignored.  */
 
 static void
-fetch_core_registers (char *core_reg_sect, unsigned core_reg_size, int which,
+fetch_core_registers (struct regcache *regcache,
+		      char *core_reg_sect, unsigned core_reg_size, int which,
 		      CORE_ADDR reg_addr)
 {
   gdb_gregset_t gregset;
   gdb_fpregset_t fpregset;
+  gdb_gregset_t *gregset_p = &gregset;
+  gdb_fpregset_t *fpregset_p = &fpregset;
 
   switch (which)
     {
@@ -73,7 +75,7 @@ fetch_core_registers (char *core_reg_sect, unsigned core_reg_size, int which,
       else
 	{
 	  memcpy (&gregset, core_reg_sect, sizeof (gregset));
-	  supply_gregset (&gregset);
+	  supply_gregset (regcache, (const gdb_gregset_t *) gregset_p);
 	}
       break;
 
@@ -83,8 +85,8 @@ fetch_core_registers (char *core_reg_sect, unsigned core_reg_size, int which,
       else
 	{
 	  memcpy (&fpregset, core_reg_sect, sizeof (fpregset));
-	  if (FP0_REGNUM >= 0)
-	    supply_fpregset (&fpregset);
+	  if (gdbarch_fp0_regnum (current_gdbarch) >= 0)
+	    supply_fpregset (regcache, (const gdb_fpregset_t *) fpregset_p);
 	}
       break;
 
