@@ -1,12 +1,12 @@
 /* Definitions for frame address handler, for GDB, the GNU debugger.
 
-   Copyright 2003, 2004 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2004, 2007 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "defs.h"
 #include "frame-base.h"
@@ -39,24 +37,12 @@ default_frame_base_address (struct frame_info *next_frame, void **this_cache)
 static CORE_ADDR
 default_frame_locals_address (struct frame_info *next_frame, void **this_cache)
 {
-  if (DEPRECATED_FRAME_LOCALS_ADDRESS_P ())
-    {
-      /* This is bad.  The computation of per-frame locals address
-	 should use a per-frame frame-base.  */
-      struct frame_info *this_frame = get_prev_frame (next_frame);
-      return DEPRECATED_FRAME_LOCALS_ADDRESS (this_frame);
-    }
   return default_frame_base_address (next_frame, this_cache);
 }
 
 static CORE_ADDR
 default_frame_args_address (struct frame_info *next_frame, void **this_cache)
 {
-  if (DEPRECATED_FRAME_ARGS_ADDRESS_P ())
-    {
-      struct frame_info *this_frame = get_prev_frame (next_frame);
-      return DEPRECATED_FRAME_ARGS_ADDRESS (this_frame);
-    }
   return default_frame_base_address (next_frame, this_cache);
 }
 
