@@ -1,12 +1,13 @@
 /* Target-dependent code for GNU/Linux running on the Fujitsu FR-V,
    for GDB.
-   Copyright (C) 2004 Free Software Foundation, Inc.
+
+   Copyright (C) 2004, 2006, 2007 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -15,9 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "defs.h"
 #include "gdbcore.h"
@@ -452,7 +451,7 @@ frv_linux_supply_fpregset (const struct regset *regset,
   regcache_raw_supply (regcache, fsr0_regnum, fpregsetp->fsr[0]);
 }
 
-/* FRV Linux register sets.  */
+/* FRV Linux kernel register sets.  */
 
 static struct regset frv_linux_gregset =
 {
