@@ -1,13 +1,12 @@
 /* Slightly more portable version of <expat.h>.
 
-   Copyright (C) 2006
-   Free Software Foundation, Inc.
+   Copyright (C) 2006, 2007 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -16,9 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #if !defined(GDB_EXPAT_H)
 #define GDB_EXPAT_H
@@ -32,4 +29,10 @@
 # define XML_STATUS_ERROR 0
 #endif
 
+/* Old versions of expat do not define this macro, so define it
+   as void.  */
+#ifndef XMLCALL
+#define XMLCALL
+#endif
+
 #endif /* !defined(GDB_EXPAT_H) */
