@@ -1,14 +1,14 @@
 /* Print values for GNU debugger GDB.
 
-   Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
-   1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995,
+   1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
    Free Software Foundation, Inc.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -17,9 +17,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "defs.h"
 #include "gdb_string.h"
@@ -70,6 +68,10 @@ static char last_size = 'w';
 
 static CORE_ADDR next_address;
 
+/* Number of delay instructions following current disassembled insn.  */
+
+static int branch_delay_insns;
+
 /* Last address examined.  */
 
 static CORE_ADDR last_examine_address;
@@ -148,45 +150,7 @@ void _initialize_printcmd (void);
 
 /* Prototypes for local functions. */
 
-static void delete_display (int);
-
-static void enable_display (char *, int);
-
-static void disable_display_command (char *, int);
-
-static void printf_command (char *, int);
-
-static void display_info (char *, int);
-
 static void do_one_display (struct display *);
-
-static void undisplay_command (char *, int);
-
-static void free_display (struct display *);
-
-static void display_command (char *, int);
-
-void x_command (char *, int);
-
-static void address_info (char *, int);
-
-static void set_command (char *, int);
-
-static void call_command (char *, int);
-
-static void inspect_command (char *, int);
-
-static void print_command (char *, int);
-
-static void print_command_1 (char *, int, int);
-
-static void validate_format (struct format_data, char *);
-
-static void print_formatted (struct value *, int, int, struct ui_file *);
-
-static struct format_data decode_format (char **, int, int);
-
-static void sym_info (char *, int);
 
 
 /* Decode a format specification.  *STRING_PTR should point to it.
@@ -250,15 +214,16 @@ decode_format (char **string_ptr, int oformat, int osize)
       case 'a':
       case 's':
 	/* Pick the appropriate size for an address.  */
-	if (TARGET_PTR_BIT == 64)
+	if (gdbarch_ptr_bit (current_gdbarch) == 64)
 	  val.size = osize ? 'g' : osize;
-	else if (TARGET_PTR_BIT == 32)
+	else if (gdbarch_ptr_bit (current_gdbarch) == 32)
 	  val.size = osize ? 'w' : osize;
-	else if (TARGET_PTR_BIT == 16)
+	else if (gdbarch_ptr_bit (current_gdbarch) == 16)
 	  val.size = osize ? 'h' : osize;
 	else
-	  /* Bad value for TARGET_PTR_BIT */
-	  internal_error (__FILE__, __LINE__, _("failed internal consistency check"));
+	  /* Bad value for gdbarch_ptr_bit.  */
+	  internal_error (__FILE__, __LINE__,
+			  _("failed internal consistency check"));
 	break;
       case 'f':
 	/* Floating point has to be word or giantword.  */
@@ -285,7 +250,8 @@ decode_format (char **string_ptr, int oformat, int osize)
    Do not end with a newline.
    0 means print VAL according to its own type.
    SIZE is the letter for the size of datum being printed.
-   This is used to pad hex numbers so they line up.  */
+   This is used to pad hex numbers so they line up.  SIZE is 0
+   for print / output and set for examine.  */
 
 static void
 print_formatted (struct value *val, int format, int size,
@@ -295,51 +261,43 @@ print_formatted (struct value *val, int format, int size,
   int len = TYPE_LENGTH (type);
 
   if (VALUE_LVAL (val) == lval_memory)
+    next_address = VALUE_ADDRESS (val) + len;
+
+  if (size)
     {
-      next_address = VALUE_ADDRESS (val) + len;
+      switch (format)
+	{
+	case 's':
+	  /* FIXME: Need to handle wchar_t's here... */
+	  next_address = VALUE_ADDRESS (val)
+	    + val_print_string (VALUE_ADDRESS (val), -1, 1, stream);
+	  return;
+
+	case 'i':
+	  /* We often wrap here if there are long symbolic names.  */
+	  wrap_here ("    ");
+	  next_address = (VALUE_ADDRESS (val)
+			  + gdb_print_insn (VALUE_ADDRESS (val), stream,
+					    &branch_delay_insns));
+	  return;
+	}
     }
 
-  switch (format)
-    {
-    case 's':
-      /* FIXME: Need to handle wchar_t's here... */
-      next_address = VALUE_ADDRESS (val)
-	+ val_print_string (VALUE_ADDRESS (val), -1, 1, stream);
-      break;
-
-    case 'i':
-      /* The old comment says
-         "Force output out, print_insn not using _filtered".
-         I'm not completely sure what that means, I suspect most print_insn
-         now do use _filtered, so I guess it's obsolete.
-         --Yes, it does filter now, and so this is obsolete.  -JB  */
-
-      /* We often wrap here if there are long symbolic names.  */
-      wrap_here ("    ");
-      next_address = VALUE_ADDRESS (val)
-	+ gdb_print_insn (VALUE_ADDRESS (val), stream);
-      break;
-
-    default:
-      if (format == 0
-	  || TYPE_CODE (type) == TYPE_CODE_ARRAY
-	  || TYPE_CODE (type) == TYPE_CODE_STRING
-	  || TYPE_CODE (type) == TYPE_CODE_STRUCT
-	  || TYPE_CODE (type) == TYPE_CODE_UNION
-	  || TYPE_CODE (type) == TYPE_CODE_NAMESPACE)
-	/* If format is 0, use the 'natural' format for
-	 * that type of value.  If the type is non-scalar,
-	 * we have to use language rules to print it as
-	 * a series of scalars.
-	 */
-	value_print (val, stream, format, Val_pretty_default);
-      else
-	/* User specified format, so don't look to the
-	 * the type to tell us what to do.
-	 */
-	print_scalar_formatted (value_contents (val), type,
-				format, size, stream);
-    }
+  if (format == 0 || format == 's'
+      || TYPE_CODE (type) == TYPE_CODE_ARRAY
+      || TYPE_CODE (type) == TYPE_CODE_STRING
+      || TYPE_CODE (type) == TYPE_CODE_STRUCT
+      || TYPE_CODE (type) == TYPE_CODE_UNION
+      || TYPE_CODE (type) == TYPE_CODE_NAMESPACE)
+    /* If format is 0, use the 'natural' format for that type of
+       value.  If the type is non-scalar, we have to use language
+       rules to print it as a series of scalars.  */
+    value_print (val, stream, format, Val_pretty_default);
+  else
+    /* User specified format, so don't look to the the type to
+       tell us what to do.  */
+    print_scalar_formatted (value_contents (val), type,
+			    format, size, stream);
 }
 
 /* Print a scalar of data of type TYPE, pointed to in GDB by VALADDR,
@@ -356,6 +314,15 @@ print_scalar_formatted (const void *valaddr, struct type *type,
   LONGEST val_long = 0;
   unsigned int len = TYPE_LENGTH (type);
 
+  /* If we get here with a string format, try again without it.  Go
+     all the way back to the language printers, which may call us
+     again.  */
+  if (format == 's')
+    {
+      val_print (type, valaddr, 0, 0, stream, 0, 0, 0, Val_pretty_default);
+      return;
+    }
+
   if (len > sizeof(LONGEST) &&
       (TYPE_CODE (type) == TYPE_CODE_INT
        || TYPE_CODE (type) == TYPE_CODE_ENUM))
@@ -388,9 +355,9 @@ print_scalar_formatted (const void *valaddr, struct type *type,
 
   /* If the value is a pointer, and pointers and addresses are not the
      same, then at this point, the value's length (in target bytes) is
-     TARGET_ADDR_BIT/TARGET_CHAR_BIT, not TYPE_LENGTH (type).  */
+     gdbarch_addr_bit/TARGET_CHAR_BIT, not TYPE_LENGTH (type).  */
   if (TYPE_CODE (type) == TYPE_CODE_PTR)
-    len = TARGET_ADDR_BIT / TARGET_CHAR_BIT;
+    len = gdbarch_addr_bit (current_gdbarch) / TARGET_CHAR_BIT;
 
   /* If we are printing it as unsigned, truncate it in case it is actually
      a negative signed value (e.g. "print/u (short)-1" should print 65535
@@ -406,7 +373,7 @@ print_scalar_formatted (const void *valaddr, struct type *type,
     case 'x':
       if (!size)
 	{
-	  /* no size specified, like in print.  Print varying # of digits. */
+	  /* No size specified, like in print.  Print varying # of digits.  */
 	  print_longest (stream, 'x', 1, val_long);
 	}
       else
@@ -446,8 +413,17 @@ print_scalar_formatted (const void *valaddr, struct type *type,
       break;
 
     case 'c':
-      value_print (value_from_longest (builtin_type_true_char, val_long),
-		   stream, 0, Val_pretty_default);
+      if (TYPE_UNSIGNED (type))
+	{
+	  struct type *utype;
+
+	  utype = builtin_type (current_gdbarch)->builtin_true_unsigned_char;
+	  value_print (value_from_longest (utype, val_long),
+		       stream, 0, Val_pretty_default);
+	}
+      else
+	value_print (value_from_longest (builtin_type_true_char, val_long),
+		     stream, 0, Val_pretty_default);
       break;
 
     case 'f':
@@ -461,7 +437,8 @@ print_scalar_formatted (const void *valaddr, struct type *type,
       break;
 
     case 0:
-      internal_error (__FILE__, __LINE__, _("failed internal consistency check"));
+      internal_error (__FILE__, __LINE__,
+		      _("failed internal consistency check"));
 
     case 't':
       /* Binary; 't' stands for "two".  */
@@ -516,7 +493,7 @@ print_scalar_formatted (const void *valaddr, struct type *type,
 }
 
 /* Specify default address for `x' command.
-   `info lines' uses this.  */
+   The `info lines' command uses this.  */
 
 void
 set_next_address (CORE_ADDR addr)
@@ -538,8 +515,8 @@ set_next_address (CORE_ADDR addr)
    settings of the demangle and asm_demangle variables.  */
 
 void
-print_address_symbolic (CORE_ADDR addr, struct ui_file *stream, int do_demangle,
-			char *leadin)
+print_address_symbolic (CORE_ADDR addr, struct ui_file *stream,
+			int do_demangle, char *leadin)
 {
   char *name = NULL;
   char *filename = NULL;
@@ -547,11 +524,12 @@ print_address_symbolic (CORE_ADDR addr, struct ui_file *stream, int do_demangle,
   int offset = 0;
   int line = 0;
 
-  /* throw away both name and filename */
+  /* Throw away both name and filename.  */
   struct cleanup *cleanup_chain = make_cleanup (free_current_contents, &name);
   make_cleanup (free_current_contents, &filename);
 
-  if (build_address_symbolic (addr, do_demangle, &name, &offset, &filename, &line, &unmapped))
+  if (build_address_symbolic (addr, do_demangle, &name, &offset,
+			      &filename, &line, &unmapped))
     {
       do_cleanups (cleanup_chain);
       return;
@@ -600,16 +578,15 @@ build_address_symbolic (CORE_ADDR addr,  /* IN */
 {
   struct minimal_symbol *msymbol;
   struct symbol *symbol;
-  struct symtab *symtab = 0;
   CORE_ADDR name_location = 0;
   asection *section = 0;
   char *name_temp = "";
   
-  /* Let's say it is unmapped. */
+  /* Let's say it is unmapped.  */
   *unmapped = 0;
 
   /* Determine if the address is in an overlay, and whether it is
-     mapped. */
+     mapped.  */
   if (overlay_debugging)
     {
       section = find_pc_overlay (addr);
@@ -648,7 +625,6 @@ build_address_symbolic (CORE_ADDR addr,  /* IN */
 	  /* The msymbol is closer to the address than the symbol;
 	     use the msymbol instead.  */
 	  symbol = 0;
-	  symtab = 0;
 	  name_location = SYMBOL_VALUE_ADDRESS (msymbol);
 	  if (do_demangle || asm_demangle)
 	    name_temp = SYMBOL_PRINT_NAME (msymbol);
@@ -685,16 +661,6 @@ build_address_symbolic (CORE_ADDR addr,  /* IN */
 	  *filename = xstrdup (sal.symtab->filename);
 	  *line = sal.line;
 	}
-      else if (symtab && symbol && symbol->line)
-	{
-	  *filename = xstrdup (symtab->filename);
-	  *line = symbol->line;
-	}
-      else if (symtab)
-	{
-	  *filename = xstrdup (symtab->filename);
-	  *line = -1;
-	}
     }
   return 0;
 }
@@ -709,7 +675,7 @@ deprecated_print_address_numeric (CORE_ADDR addr, int use_local,
     fputs_filtered (paddress (addr), stream);
   else
     {
-      int addr_bit = TARGET_ADDR_BIT;
+      int addr_bit = gdbarch_addr_bit (current_gdbarch);
 
       if (addr_bit < (sizeof (CORE_ADDR) * HOST_CHAR_BIT))
 	addr &= ((CORE_ADDR) 1 << addr_bit) - 1;
@@ -734,7 +700,8 @@ print_address (CORE_ADDR addr, struct ui_file *stream)
    or not.  */
 
 void
-print_address_demangle (CORE_ADDR addr, struct ui_file *stream, int do_demangle)
+print_address_demangle (CORE_ADDR addr, struct ui_file *stream,
+			int do_demangle)
 {
   if (addr == 0)
     {
@@ -825,8 +792,8 @@ do_examine (struct format_data fmt, CORE_ADDR addr)
 	    value_free (last_examine_value);
 
 	  /* The value to be displayed is not fetched greedily.
-	     Instead, to avoid the posibility of a fetched value not
-	     being used, its retreval is delayed until the print code
+	     Instead, to avoid the possibility of a fetched value not
+	     being used, its retrieval is delayed until the print code
 	     uses it.  When examining an instruction stream, the
 	     disassembler will perform its own memory fetch using just
 	     the address stored in LAST_EXAMINE_VALUE.  FIXME: Should
@@ -839,6 +806,10 @@ do_examine (struct format_data fmt, CORE_ADDR addr)
 	    release_value (last_examine_value);
 
 	  print_formatted (last_examine_value, format, size, gdb_stdout);
+
+	  /* Display any branch delay slots following the final insn.  */
+	  if (format == 'i' && count == 1)
+	    count += branch_delay_insns;
 	}
       printf_filtered ("\n");
       gdb_flush (gdb_stdout);
@@ -853,15 +824,14 @@ validate_format (struct format_data fmt, char *cmdname)
   if (fmt.count != 1)
     error (_("Item count other than 1 is meaningless in \"%s\" command."),
 	   cmdname);
-  if (fmt.format == 'i' || fmt.format == 's')
+  if (fmt.format == 'i')
     error (_("Format letter \"%c\" is meaningless in \"%s\" command."),
 	   fmt.format, cmdname);
 }
 
-/*  Evaluate string EXP as an expression in the current language and
+/* Evaluate string EXP as an expression in the current language and
    print the resulting value.  EXP may contain a format specifier as the
-   first argument ("/x myvar" for example, to print myvar in hex).
- */
+   first argument ("/x myvar" for example, to print myvar in hex).  */
 
 static void
 print_command_1 (char *exp, int inspect, int voidprint)
@@ -873,7 +843,8 @@ print_command_1 (char *exp, int inspect, int voidprint)
   struct format_data fmt;
   int cleanup = 0;
 
-  /* Pass inspect flag to the rest of the print routines in a global (sigh). */
+  /* Pass inspect flag to the rest of the print routines in a global
+     (sigh).  */
   inspect_it = inspect;
 
   if (exp && *exp == '/')
@@ -912,7 +883,8 @@ print_command_1 (char *exp, int inspect, int voidprint)
 	annotate_value_begin (value_type (val));
 
       if (inspect)
-	printf_unfiltered ("\031(gdb-makebuffer \"%s\"  %d '(\"", exp, histindex);
+	printf_unfiltered ("\031(gdb-makebuffer \"%s\"  %d '(\"",
+			   exp, histindex);
       else if (histindex >= 0)
 	printf_filtered ("$%d = ", histindex);
 
@@ -933,7 +905,7 @@ print_command_1 (char *exp, int inspect, int voidprint)
 
   if (cleanup)
     do_cleanups (old_chain);
-  inspect_it = 0;		/* Reset print routines to normal */
+  inspect_it = 0;		/* Reset print routines to normal.  */
 }
 
 static void
@@ -942,7 +914,7 @@ print_command (char *exp, int from_tty)
   print_command_1 (exp, 0, 1);
 }
 
-/* Same as print, except in epoch, it gets its own window */
+/* Same as print, except in epoch, it gets its own window.  */
 static void
 inspect_command (char *exp, int from_tty)
 {
@@ -951,7 +923,7 @@ inspect_command (char *exp, int from_tty)
   print_command_1 (exp, epoch_interface, 1);
 }
 
-/* Same as print, except it doesn't print void results. */
+/* Same as print, except it doesn't print void results.  */
 static void
 call_command (char *exp, int from_tty)
 {
@@ -1021,6 +993,11 @@ sym_info (char *arg, int from_tty)
   addr = parse_and_eval_address (arg);
   ALL_OBJSECTIONS (objfile, osect)
   {
+    /* Only process each object file once, even if there's a separate
+       debug file.  */
+    if (objfile->separate_debug_objfile_backlink)
+      continue;
+
     sect = osect->the_bfd_section;
     sect_addr = overlay_mapped_address (addr, sect);
 
@@ -1146,7 +1123,8 @@ address_info (char *exp, int from_tty)
       break;
 
     case LOC_REGISTER:
-      printf_filtered (_("a variable in register %s"), REGISTER_NAME (val));
+      printf_filtered (_("a variable in register %s"),
+			 gdbarch_register_name (current_gdbarch, val));
       break;
 
     case LOC_STATIC:
@@ -1177,11 +1155,13 @@ address_info (char *exp, int from_tty)
       break;
 
     case LOC_REGPARM:
-      printf_filtered (_("an argument in register %s"), REGISTER_NAME (val));
+      printf_filtered (_("an argument in register %s"),
+			 gdbarch_register_name (current_gdbarch, val));
       break;
 
     case LOC_REGPARM_ADDR:
-      printf_filtered (_("address of an argument in register %s"), REGISTER_NAME (val));
+      printf_filtered (_("address of an argument in register %s"),
+		       gdbarch_register_name (current_gdbarch, val));
       break;
 
     case LOC_ARG:
@@ -1202,12 +1182,12 @@ address_info (char *exp, int from_tty)
 
     case LOC_BASEREG:
       printf_filtered (_("a variable at offset %ld from register %s"),
-		       val, REGISTER_NAME (basereg));
+		       val, gdbarch_register_name (current_gdbarch, basereg));
       break;
 
     case LOC_BASEREG_ARG:
       printf_filtered (_("an argument at offset %ld from register %s"),
-		       val, REGISTER_NAME (basereg));
+		       val, gdbarch_register_name (current_gdbarch, basereg));
       break;
 
     case LOC_TYPEDEF:
@@ -1216,8 +1196,8 @@ address_info (char *exp, int from_tty)
 
     case LOC_BLOCK:
       printf_filtered (_("a function at address "));
-      deprecated_print_address_numeric (load_addr = BLOCK_START (SYMBOL_BLOCK_VALUE (sym)),
-			     1, gdb_stdout);
+      load_addr = BLOCK_START (SYMBOL_BLOCK_VALUE (sym));
+      deprecated_print_address_numeric (load_addr, 1, gdb_stdout);
       if (section_is_overlay (section))
 	{
 	  load_addr = overlay_unmapped_address (load_addr, section);
@@ -1238,8 +1218,8 @@ address_info (char *exp, int from_tty)
 	  {
 	    section = SYMBOL_BFD_SECTION (msym);
 	    printf_filtered (_("static storage at address "));
-	    deprecated_print_address_numeric (load_addr = SYMBOL_VALUE_ADDRESS (msym),
-				   1, gdb_stdout);
+	    load_addr = SYMBOL_VALUE_ADDRESS (msym);
+	    deprecated_print_address_numeric (load_addr, 1, gdb_stdout);
 	    if (section_is_overlay (section))
 	      {
 		load_addr = overlay_unmapped_address (load_addr, section);
@@ -1252,9 +1232,9 @@ address_info (char *exp, int from_tty)
       break;
 
     case LOC_HP_THREAD_LOCAL_STATIC:
-      printf_filtered (
-			"a thread-local variable at offset %ld from the thread base register %s",
-			val, REGISTER_NAME (basereg));
+      printf_filtered (_("\
+a thread-local variable at offset %ld from the thread base register %s"),
+		       val, gdbarch_register_name (current_gdbarch, basereg));
       break;
 
     case LOC_OPTIMIZED_OUT:
@@ -1268,7 +1248,8 @@ address_info (char *exp, int from_tty)
   printf_filtered (".\n");
 }
 
-void
+
+static void
 x_command (char *exp, int from_tty)
 {
   struct expression *expr;
@@ -1291,9 +1272,9 @@ x_command (char *exp, int from_tty)
   if (exp != 0 && *exp != 0)
     {
       expr = parse_expression (exp);
-      /* Cause expression not to be there any more
-         if this command is repeated with Newline.
-         But don't clobber a user-defined command's definition.  */
+      /* Cause expression not to be there any more if this command is
+         repeated with Newline.  But don't clobber a user-defined
+         command's definition.  */
       if (from_tty)
 	*exp = 0;
       old_chain = make_cleanup (free_current_contents, &expr);
@@ -1313,7 +1294,8 @@ x_command (char *exp, int from_tty)
 
   do_examine (fmt, next_address);
 
-  /* If the examine succeeds, we remember its size and format for next time.  */
+  /* If the examine succeeds, we remember its size and format for next
+     time.  */
   last_size = fmt.size;
   last_format = fmt.format;
 
@@ -1328,9 +1310,10 @@ x_command (char *exp, int from_tty)
 		       value_from_pointer (pointer_type,
 					   last_examine_address));
 
-      /* Make contents of last address examined available to the user as $__. */
-      /* If the last value has not been fetched from memory then don't
-         fetch it now - instead mark it by voiding the $__ variable. */
+      /* Make contents of last address examined available to the user
+	 as $__.  If the last value has not been fetched from memory
+	 then don't fetch it now; instead mark it by voiding the $__
+	 variable.  */
       if (value_lazy (last_examine_value))
 	set_internalvar (lookup_internalvar ("__"),
 			 allocate_value (builtin_type_void));
@@ -1409,9 +1392,8 @@ free_display (struct display *d)
   xfree (d);
 }
 
-/* Clear out the display_chain.
-   Done when new symtabs are loaded, since this invalidates
-   the types stored in many expressions.  */
+/* Clear out the display_chain.  Done when new symtabs are loaded,
+   since this invalidates the types stored in many expressions.  */
 
 void
 clear_displays (void)
@@ -1539,7 +1521,7 @@ do_one_display (struct display *d)
       print_expression (d->exp, gdb_stdout);
       annotate_display_expression_end ();
 
-      if (d->format.count != 1)
+      if (d->format.count != 1 || d->format.format == 'i')
 	printf_filtered ("\n");
       else
 	printf_filtered ("  ");
@@ -1547,7 +1529,7 @@ do_one_display (struct display *d)
       val = evaluate_expression (d->exp);
       addr = value_as_address (val);
       if (d->format.format == 'i')
-	addr = ADDR_BITS_REMOVE (addr);
+	addr = gdbarch_addr_bits_remove (current_gdbarch, addr);
 
       annotate_display_value ();
 
@@ -1615,7 +1597,8 @@ disable_current_display (void)
   if (current_display_number >= 0)
     {
       disable_display (current_display_number);
-      fprintf_unfiltered (gdb_stderr, "Disabling display %d to avoid infinite recursion.\n",
+      fprintf_unfiltered (gdb_stderr, _("\
+Disabling display %d to avoid infinite recursion.\n"),
 			  current_display_number);
     }
   current_display_number = -1;
@@ -1716,8 +1699,8 @@ disable_display_command (char *args, int from_tty)
 }
 
 
-/* Print the value in stack frame FRAME of a variable
-   specified by a struct symbol.  */
+/* Print the value in stack frame FRAME of a variable specified by a
+   struct symbol.  */
 
 void
 print_variable_value (struct symbol *var, struct frame_info *frame,
@@ -1741,8 +1724,7 @@ printf_command (char *arg, int from_tty)
   int allocated_args = 20;
   struct cleanup *old_cleanups;
 
-  val_args = (struct value **) xmalloc (allocated_args
-					* sizeof (struct value *));
+  val_args = xmalloc (allocated_args * sizeof (struct value *));
   old_cleanups = make_cleanup (free_current_contents, &val_args);
 
   if (s == 0)
@@ -1752,7 +1734,7 @@ printf_command (char *arg, int from_tty)
   while (*s == ' ' || *s == '\t')
     s++;
 
-  /* A format string should follow, enveloped in double quotes */
+  /* A format string should follow, enveloped in double quotes.  */
   if (*s++ != '"')
     error (_("Bad format string, missing '\"'."));
 
@@ -2112,8 +2094,68 @@ printf_command (char *arg, int from_tty)
 	      printf_filtered (current_substring, val);
 	      break;
 	    }
-	  default:		/* purecov: deadcode */
-	    error (_("internal error in printf_command"));		/* purecov: deadcode */
+	  case ptr_arg:
+	    {
+	      /* We avoid the host's %p because pointers are too
+		 likely to be the wrong size.  The only interesting
+		 modifier for %p is a width; extract that, and then
+		 handle %p as glibc would: %#x or a literal "(nil)".  */
+
+	      char *p, *fmt, *fmt_p;
+#if defined (CC_HAS_LONG_LONG) && defined (PRINTF_HAS_LONG_LONG)
+	      long long val = value_as_long (val_args[i]);
+#else
+	      long val = value_as_long (val_args[i]);
+#endif
+
+	      fmt = alloca (strlen (current_substring) + 5);
+
+	      /* Copy up to the leading %.  */
+	      p = current_substring;
+	      fmt_p = fmt;
+	      while (*p)
+		{
+		  int is_percent = (*p == '%');
+		  *fmt_p++ = *p++;
+		  if (is_percent)
+		    {
+		      if (*p == '%')
+			*fmt_p++ = *p++;
+		      else
+			break;
+		    }
+		}
+
+	      if (val != 0)
+		*fmt_p++ = '#';
+
+	      /* Copy any width.  */
+	      while (*p >= '0' && *p < '9')
+		*fmt_p++ = *p++;
+
+	      gdb_assert (*p == 'p' && *(p + 1) == '\0');
+	      if (val != 0)
+		{
+#if defined (CC_HAS_LONG_LONG) && defined (PRINTF_HAS_LONG_LONG)
+		  *fmt_p++ = 'l';
+#endif
+		  *fmt_p++ = 'l';
+		  *fmt_p++ = 'x';
+		  *fmt_p++ = '\0';
+		  printf_filtered (fmt, val);
+		}
+	      else
+		{
+		  *fmt_p++ = 's';
+		  *fmt_p++ = '\0';
+		  printf_filtered (fmt, "(nil)");
+		}
+
+	      break;
+	    }
+	  default:
+	    internal_error (__FILE__, __LINE__,
+			    _("failed internal consistency check"));
 	  }
 	/* Skip to the next substring.  */
 	current_substring += strlen (current_substring) + 1;
