@@ -1,12 +1,12 @@
 /* Target-dependent code for UltraSPARC.
 
-   Copyright (C) 2003, 2004 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2004, 2006, 2007 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifndef SPARC64_TDEP_H
 #define SPARC64_TDEP_H 1
@@ -84,6 +82,18 @@ enum sparc64_regnum
   = SPARC64_Q0_REGNUM + 15
 };
 
+/* Processor state bits.  */
+#define SPARC64_PSTATE_AG	0x001
+#define SPARC64_PSTATE_IE	0x002
+#define SPARC64_PSTATE_PRIV	0x004
+#define SPARC64_PSTATE_AM	0x008
+#define SPARC64_PSTATE_PEF	0x010
+#define SPARC64_PSTATE_RED	0x020
+#define SPARC64_PSTATE_TLE	0x100
+#define SPARC64_PSTATE_CLE	0x200
+#define SPARC64_PSTATE_PID0	0x400
+#define SPARC64_PSTATE_PID1	0x800
+
 extern void sparc64_init_abi (struct gdbarch_info info,
 			      struct gdbarch *gdbarch);
 
