@@ -1,7 +1,7 @@
 /* Specific command window processing.
 
-   Copyright 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005 Free Software
-   Foundation, Inc.
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007
+   Free Software Foundation, Inc.
 
    Contributed by Hewlett-Packard Company.
 
@@ -9,7 +9,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -18,9 +18,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "defs.h"
 #include <ctype.h>
@@ -43,7 +41,8 @@
 ** PUBLIC FUNCTIONS                        **
 ******************************************/
 
-/* Dispatch the correct tui function based upon the control character.   */
+/* Dispatch the correct tui function based upon the control
+   character.  */
 unsigned int
 tui_dispatch_ctrl_char (unsigned int ch)
 {
@@ -54,8 +53,8 @@ tui_dispatch_ctrl_char (unsigned int ch)
     tui_refresh_all_win ();
 
   /* If the command window has the logical focus, or no-one does
-     assume it is the command window; in this case, pass the
-     character on through and do nothing here.  */
+     assume it is the command window; in this case, pass the character
+     on through and do nothing here.  */
   if (win_info == NULL || win_info == TUI_CMD_WIN)
     return ch;
   else
@@ -64,16 +63,16 @@ tui_dispatch_ctrl_char (unsigned int ch)
       int i;
       char *term;
 
-      /* If this is an xterm, page next/prev keys aren't returned
-         ** by keypad as a single char, so we must handle them here.
-         ** Seems like a bug in the curses library?
-       */
+      /* If this is an xterm, page next/prev keys aren't returned by
+         keypad as a single char, so we must handle them here.  Seems
+         like a bug in the curses library?  */
       term = (char *) getenv ("TERM");
       if (term)
 	{
 	  for (i = 0; term[i]; i++)
 	    term[i] = toupper (term[i]);
-	  if ((strcmp (term, "XTERM") == 0) && key_is_start_sequence (ch))
+	  if ((strcmp (term, "XTERM") == 0) 
+	      && key_is_start_sequence (ch))
 	    {
 	      unsigned int page_ch = 0;
 	      unsigned int tmp_char;
