@@ -1,7 +1,7 @@
 /* Data/register window display.
 
-   Copyright 1998, 1999, 2000, 2001, 2002, 2003, 2004 Free Software
-   Foundation, Inc.
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2007
+   Free Software Foundation, Inc.
 
    Contributed by Hewlett-Packard Company.
 
@@ -9,7 +9,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -18,9 +18,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "defs.h"
 #include "tui/tui.h"
@@ -51,13 +49,16 @@ tui_first_data_item_displayed (void)
   int element_no = (-1);
   int i;
 
-  for (i = 0; (i < TUI_DATA_WIN->generic.content_size && element_no < 0); i++)
+  for (i = 0; 
+       i < TUI_DATA_WIN->generic.content_size && element_no < 0;
+       i++)
     {
-      struct tui_gen_win_info * data_item_win;
+      struct tui_gen_win_info *data_item_win;
 
       data_item_win = &((tui_win_content)
-		      TUI_DATA_WIN->generic.content)[i]->which_element.data_window;
-      if (data_item_win->handle != (WINDOW *) NULL && data_item_win->is_visible)
+			TUI_DATA_WIN->generic.content)[i]->which_element.data_window;
+      if (data_item_win->handle != (WINDOW *) NULL 
+	  && data_item_win->is_visible)
 	element_no = i;
     }
 
@@ -72,14 +73,10 @@ tui_first_data_element_no_in_line (int line_no)
 {
   int first_element_no = (-1);
 
-  /*
-     ** First see if there is a register on line_no, and if so, set the
-     ** first element number
-   */
+  /* First see if there is a register on line_no, and if so, set the
+     first element number.  */
   if ((first_element_no = tui_first_reg_element_no_inline (line_no)) == -1)
-    {				/*
-				   ** Looking at the general data, the 1st element on line_no
-				 */
+    { /* Looking at the general data, the 1st element on line_no.  */
     }
 
   return first_element_no;
@@ -92,12 +89,12 @@ void
 tui_delete_data_content_windows (void)
 {
   int i;
-  struct tui_gen_win_info * data_item_win_ptr;
+  struct tui_gen_win_info *data_item_win_ptr;
 
   for (i = 0; (i < TUI_DATA_WIN->generic.content_size); i++)
     {
       data_item_win_ptr = &((tui_win_content)
-		      TUI_DATA_WIN->generic.content)[i]->which_element.data_window;
+			    TUI_DATA_WIN->generic.content)[i]->which_element.data_window;
       tui_delete_win (data_item_win_ptr->handle);
       data_item_win_ptr->handle = (WINDOW *) NULL;
       data_item_win_ptr->is_visible = FALSE;
@@ -141,12 +138,11 @@ tui_display_all_data (void)
       tui_delete_data_content_windows ();
       tui_check_and_display_highlight_if_needed (TUI_DATA_WIN);
       tui_display_registers_from (0);
-      /*
-         ** Then display the other data
-       */
+
+      /* Then display the other data.  */
       if (TUI_DATA_WIN->detail.data_display_info.data_content !=
-	  (tui_win_content) NULL &&
-	  TUI_DATA_WIN->detail.data_display_info.data_content_count > 0)
+	  (tui_win_content) NULL 
+	  && TUI_DATA_WIN->detail.data_display_info.data_content_count > 0)
 	{
 	}
     }
@@ -165,7 +161,8 @@ tui_display_data_from_line (int line_no)
 
   tui_check_and_display_highlight_if_needed (TUI_DATA_WIN);
 
-  /* there is no general data, force regs to display (if there are any) */
+  /* There is no general data, force regs to display (if there are
+     any).  */
   if (TUI_DATA_WIN->detail.data_display_info.data_content_count <= 0)
     tui_display_registers_from_line (_line_no, TRUE);
   else
@@ -174,30 +171,26 @@ tui_display_data_from_line (int line_no)
       int regs_last_line = tui_last_regs_line_no ();
 
 
-      /* display regs if we can */
+      /* Display regs if we can.  */
       if (tui_display_registers_from_line (_line_no, FALSE) < 0)
-	{			/*
-				   ** _line_no is past the regs display, so calc where the
-				   ** start data element is
-				 */
+	{ /* _line_no is past the regs display, so calc where the
+	     start data element is.  */
 	  if (regs_last_line < _line_no)
-	    {			/* figure out how many lines each element is to obtain
-				   the start element_no */
+	    { /* Figure out how many lines each element is to obtain
+		 the start element_no.  */
 	    }
 	}
       else
-	{			/*
-				   ** calculate the starting element of the data display, given
-				   ** regs_last_line and how many lines each element is, up to
-				   ** _line_no
-				 */
+	{ /* Calculate the starting element of the data display, given
+	     regs_last_line and how many lines each element is, up to
+	     _line_no.  */
 	}
-      /* Now display the data , starting at element_no */
+      /* Now display the data , starting at element_no.  */
     }
 }
 
 
-/* Display data starting at element element_no.   */
+/* Display data starting at element element_no.  */
 void
 tui_display_data_from (int element_no, int reuse_windows)
 {
@@ -206,7 +199,7 @@ tui_display_data_from (int element_no, int reuse_windows)
   if (element_no < TUI_DATA_WIN->detail.data_display_info.regs_content_count)
     first_line = tui_line_from_reg_element_no (element_no);
   else
-    {				/* calculate the first_line from the element number */
+    { /* Calculate the first_line from the element number.  */
     }
 
   if (first_line >= 0)
@@ -228,28 +221,31 @@ tui_refresh_data_win (void)
     {
       int first_element = tui_first_data_item_displayed ();
 
-      if (first_element >= 0)	/* re-use existing windows */
+      if (first_element >= 0)	/* Re-use existing windows.  */
 	tui_display_data_from (first_element, TRUE);
     }
 }
 
 
-/* Function to check the data values and hilite any that have changed.  */
+/* Function to check the data values and hilite any that have
+   changed.  */
 void
 tui_check_data_values (struct frame_info *frame)
 {
   tui_check_register_values (frame);
 
-  /* Now check any other data values that there are */
+  /* Now check any other data values that there are.  */
   if (TUI_DATA_WIN != NULL && TUI_DATA_WIN->generic.is_visible)
     {
       int i;
 
-      for (i = 0; TUI_DATA_WIN->detail.data_display_info.data_content_count; i++)
+      for (i = 0; 
+	   TUI_DATA_WIN->detail.data_display_info.data_content_count; 
+	   i++)
 	{
 #ifdef LATER
 	  tui_data_element_ptr data_element_ptr;
-	  struct tui_gen_win_info * data_item_win_ptr;
+	  struct tui_gen_win_info *data_item_win_ptr;
 	  Opaque new_value;
 
 	  data_item_ptr = &TUI_DATA_WIN->detail.data_display_info.
@@ -268,9 +264,10 @@ tui_check_data_values (struct frame_info *frame)
 }
 
 
-/* Scroll the data window vertically forward or backward.   */
+/* Scroll the data window vertically forward or backward.  */
 void
-tui_vertical_data_scroll (enum tui_scroll_direction scroll_direction, int num_to_scroll)
+tui_vertical_data_scroll (enum tui_scroll_direction scroll_direction,
+			  int num_to_scroll)
 {
   int first_element_no;
   int first_line = (-1);
@@ -279,9 +276,8 @@ tui_vertical_data_scroll (enum tui_scroll_direction scroll_direction, int num_to
   if (first_element_no < TUI_DATA_WIN->detail.data_display_info.regs_content_count)
     first_line = tui_line_from_reg_element_no (first_element_no);
   else
-    {				/* calculate the first line from the element number which is in
-				   ** the general data content
-				 */
+    { /* Calculate the first line from the element number which is in
+        the general data content.  */
     }
 
   if (first_line >= 0)
