@@ -1,12 +1,12 @@
 /* Language independent support for printing types for GDB, the GNU debugger.
-   Copyright (C) 1986, 1988, 1989, 1991-1993, 1999, 2000
+   Copyright (C) 1986, 1988, 1989, 1991-1993, 1999, 2000, 2007
    Free Software Foundation, Inc.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifndef TYPEPRINT_H
 #define TYPEPRINT_H
